@@ -1,4 +1,5 @@
 #!/bin/bash
+
 if [[ "$(uname)" == Darwin ]]; then
   # Uninstall Conflicting jpeg brew formulae
   jpeg_packages=$(brew list | grep jpeg)
@@ -12,8 +13,10 @@ if [[ "$(uname)" == Darwin ]]; then
 fi
 
 if [[ "$(uname)" == Darwin || "$OSTYPE" == "msys" ]]; then
-  # Install libpng from Anaconda (defaults)
-  conda install libpng -yq
+  conda install libpng libwebp -yq
+  # Installing webp also installs a non-turbo jpeg, so we uninstall jpeg stuff
+  # before re-installing them
+  conda uninstall libjpeg-turbo libjpeg -y
   conda install -yq ffmpeg=4.2 libjpeg-turbo -c pytorch
 
   # Copy binaries to be included in the wheel distribution
@@ -29,21 +32,8 @@ else
     conda install -yq ffmpeg=4.2 libjpeg-turbo -c pytorch-nightly
   fi
 
-  # Install native CentOS libJPEG, freetype and GnuTLS
-  yum install -y libjpeg-turbo-devel freetype gnutls
-
-  # Download all the dependencies required to compile image and video_reader
-  # extensions
-  mkdir -p ext_libraries
-  pushd ext_libraries
-  popd
-  export PATH="$(pwd)/ext_libraries/bin:$PATH"
+  yum install -y libjpeg-turbo-devel libwebp-devel freetype gnutls
   pip install auditwheel
-
-  # Point to custom libraries
-  export LD_LIBRARY_PATH=$(pwd)/ext_libraries/lib:$LD_LIBRARY_PATH
-  export TORCHVISION_INCLUDE=$(pwd)/ext_libraries/include
-  export TORCHVISION_LIBRARY=$(pwd)/ext_libraries/lib
 fi
 
 pip install numpy pyyaml future ninja
