@@ -1,9 +1,13 @@
 #pragma once
 
+#include "cpu/decode_avif.h"
+#include "cpu/decode_gif.h"
+#include "cpu/decode_heic.h"
 #include "cpu/decode_image.h"
 #include "cpu/decode_jpeg.h"
 #include "cpu/decode_png.h"
+#include "cpu/decode_webp.h"
 #include "cpu/encode_jpeg.h"
 #include "cpu/encode_png.h"
 #include "cpu/read_write_file.h"
-#include "cuda/decode_jpeg_cuda.h"
+#include "cuda/encode_decode_jpegs_cuda.h"
